// Arduino UNO OLED Example demo code
//              Introduction: 
//              ----------------------------------------------------------------
//              GND  GND
//              VCC  3.3v 
//              D0   8（SCL）
//              D1   9（SDA）                    
//              ----------------------------------------------------------------
//******************************************************************************/
#include "font.h"

int scl=8; 
int sda=9; 

#define OLED_SCLK_Clr() digitalWrite(scl,LOW)//SCL
#define OLED_SCLK_Set() digitalWrite(scl,HIGH)

#define OLED_SDIN_Clr() digitalWrite(sda,LOW)//SDA
#define OLED_SDIN_Set() digitalWrite(sda,HIGH)

#define OLED_CMD  0  //write command
#define OLED_DATA 1 //write data

uint8_t OLED_GRAM[128][4]; 

void setup()
{
  uint8_t t=' ';
  OLED_Init();
  OLED_ColorTurn(0);   //0 normal color ,1 reverse color
  OLED_DisplayTurn(0); //0 mormal color, 1 rotate 180 degree
  while(1)
  {
    OLED_ShowPicture(0,0,128,4,BMP1);
    delay(1500);
    OLED_Clear();
    OLED_ShowChinese(18,0,1,16);
    OLED_ShowChinese(18,0,1,16);
    OLED_ShowChinese(36,0,2,16);
    OLED_ShowChinese(54,0,3,16);
    OLED_ShowChinese(72,0,4,16);
    OLED_ShowChinese(90,0,5,16);
    OLED_ShowChinese(108,0,6,16);
    OLED_ShowString(8,0,"52Pi-Technology",16);
    OLED_ShowString(8,16,"52Pi-Technology",16);
    OLED_Refresh();
    delay(5000);
    OLED_Clear();
    OLED_ShowString(20,0,"2019/12/09",16);
    OLED_ShowString(0,16,"ASCII:",16);  
    OLED_ShowString(63,16,"OLED:",16);
    OLED_ShowChar(48,16,t,16);//display ASCII characters  
    t++;
    if(t>'~')t=' ';
    OLED_ShowNum(103,16,t,3,16);
    OLED_Refresh();
    delay(5000);
    OLED_Clear();
    OLED_ShowChinese(16,0,0,16); //16*16 
    OLED_ShowChinese(32,0,0,24);//24*24  
    OLED_ShowChinese(64,0,0,32);//32*32  
    OLED_Refresh();
    delay(5000);
    OLED_Clear();
    OLED_ShowString(0,0,"52Pi",12);
    OLED_ShowString(18,0,"52Pi",16);
    OLED_ShowString(42,0,"52Pi",24);
    OLED_Refresh();
    delay(5000);
  }
}

void loop()
{
}
//reverse  function 
void OLED_ColorTurn(u8 i)
{
  if(!i) OLED_WR_Byte(0xA6,OLED_CMD);//Normal Display
  else  OLED_WR_Byte(0xA7,OLED_CMD);//Reverse Display
}

//rotate 180 degree
void OLED_DisplayTurn(u8 i)
{
  if(i==0)
    {
      OLED_WR_Byte(0xC8,OLED_CMD);//Normal Display
      OLED_WR_Byte(0xA1,OLED_CMD);
    }
else
    {
      OLED_WR_Byte(0xC0,OLED_CMD);//Reverse Display
      OLED_WR_Byte(0xA0,OLED_CMD);
    }
}
//Initializing
void I2C_Start(void)
{
  OLED_SDIN_Set();
  OLED_SCLK_Set();
  OLED_SDIN_Clr();
  OLED_SCLK_Clr();
}

//End signal
void I2C_Stop(void)
{
  OLED_SCLK_Set();
  OLED_SDIN_Clr();
  OLED_SDIN_Set();
}

//wait signal 
void I2C_WaitAck(void) //detect data signal level 
{
  OLED_SCLK_Set();
  OLED_SCLK_Clr();
}

//write a byte
void Send_Byte(uint8_t dat)
{
  uint8_t i;
  for(i=0;i<8;i++)
  {
    OLED_SCLK_Clr();//pull down clock signal level  
    if(dat&0x80)//check data and set data bit
    {
      OLED_SDIN_Set();
    }
    else
    {
      OLED_SDIN_Clr();
    }
    OLED_SCLK_Set();//pull up signal level  
    OLED_SCLK_Clr();//pull down clock signal level
    dat<<=1; 
  }
}

//send a byte to SSD1306 chip.
//mode:data/command signal 0,command;1,data;
void OLED_WR_Byte(uint8_t dat,uint8_t mode)
{
  I2C_Start();
  Send_Byte(0x78);
  I2C_WaitAck();
  if(mode){Send_Byte(0x40);}
  else{Send_Byte(0x00);}
  I2C_WaitAck();
  Send_Byte(dat);
  I2C_WaitAck();
  I2C_Stop();
}

//update buffer to OLED  
void OLED_Refresh(void)
{
  uint8_t i,n;
  for(i=0;i<4;i++)
  {
     OLED_WR_Byte(0xb0+i,OLED_CMD); //set herizontal line start address
     OLED_WR_Byte(0x00,OLED_CMD);   //set low vertical line start address
     OLED_WR_Byte(0x10,OLED_CMD);   //set high vertical line start address
     for(n=0;n<128;n++)
     OLED_WR_Byte(OLED_GRAM[n][i],OLED_DATA);
  }
}
//clear 
void OLED_Clear(void)
{
  uint8_t i,n;
  for(i=0;i<4;i++)
  {
     for(n=0;n<128;n++)
      {
       OLED_GRAM[n][i]=0;//clear up all
      }
  }
  OLED_Refresh();//update 
}

//draw point 
//x:0~127
//y:0~63
void OLED_DrawPoint(uint8_t x,uint8_t y)
{
  uint8_t i,m,n;
  i=y/8;
  m=y%8;
  n=1<<m;
  OLED_GRAM[x][i]|=n;
}

//clear point
//x:0~127
//y:0~63
void OLED_ClearPoint(uint8_t x,uint8_t y)
{
  uint8_t i,m,n;
  i=y/8;
  m=y%8;
  n=1<<m;
  OLED_GRAM[x][i]=~OLED_GRAM[x][i];
  OLED_GRAM[x][i]|=n;
  OLED_GRAM[x][i]=~OLED_GRAM[x][i];
}

//draw line
//x:0~128
//y:0~64
void OLED_DrawLine(uint8_t x1,uint8_t y1,uint8_t x2,uint8_t y2)
{
  uint8_t i,k,k1,k2,y0;
  if(x1==x2)    //draw vertial line
  {
      for(i=0;i<(y2-y1);i++)
      {
        OLED_DrawPoint(x1,y1+i);
      }
  }
  else if(y1==y2)   //draw horizontal line
  {
      for(i=0;i<(x2-x1);i++)
      {
        OLED_DrawPoint(x1+i,y1);
      }
  }
  else      //draw slashes
  {
    k1=y2-y1;
    k2=x2-x1;
    k=k1*10/k2;
    for(i=0;i<(x2-x1);i++)
      {
        OLED_DrawPoint(x1+i,y1+i*k/10);
      }
  }
}
//x,y:Center coordinates
//r:Radius of the circle
void OLED_DrawCircle(uint8_t x,uint8_t y,uint8_t r)
{
  int a, b,num;
    a = 0;
    b = r;
    while(2 * b * b >= r * r)      
    {
        OLED_DrawPoint(x + a, y - b);
        OLED_DrawPoint(x - a, y - b);
        OLED_DrawPoint(x - a, y + b);
        OLED_DrawPoint(x + a, y + b);
 
        OLED_DrawPoint(x + b, y + a);
        OLED_DrawPoint(x + b, y - a);
        OLED_DrawPoint(x - b, y - a);
        OLED_DrawPoint(x - b, y + a);
        
        a++;
        num = (a * a + b * b) - r*r;//Calculate the distance of the point of the painting from the center of the circle
        if(num > 0)
        {
            b--;
            a--;
        }
    }
}

//Show a character, including some characters, at the specified location
//x:0~127
//y:0~63
//size:Select a font 12/16/24
//Modeling mode, column-by-column 
void OLED_ShowChar(uint8_t x,uint8_t y,const char chr,uint8_t size1)
{
  uint8_t i,m,temp,size2,chr1;
  uint8_t y0=y;
  size2=(size1/8+((size1%8)?1:0))*(size1/2);  //Get the number of bytes that a font character corresponds to a set of dot matrixes
  chr1=chr-' ';  //Calculating the value after the offset
  for(i=0;i<size2;i++)
  {
    if(size1==12)
        { 
          temp=pgm_read_byte(&asc2_1206[chr1][i]);
        } //Call 1206 font
    else if(size1==16)
        {
          temp=pgm_read_byte(&asc2_1608[chr1][i]);
        } //Call 1608 font
    else if(size1==24)
        {
          temp=pgm_read_byte(&asc2_2412[chr1][i]);
        } //Call 2412 font
    else return;
        for(m=0;m<8;m++)           //Write data
        {
          if(temp&0x80)OLED_DrawPoint(x,y);
          else OLED_ClearPoint(x,y);
          temp<<=1;
          y++;
          if((y-y0)==size1)
          {
            y=y0;
            x++;
            break;
          }
        }
  }
}


//Show string
//x,y:Start coordinates 
//size1:Font size 
//*chr:String start address
void OLED_ShowString(uint8_t x,uint8_t y,const char *chr,uint8_t size1)
{
  while((*chr>=' ')&&(*chr<='~'))//to determine if it's illegal characters
  {
    OLED_ShowChar(x,y,*chr,size1);
    x+=size1/2;
    if(x>128-size1/2)  //Line breaks
    {
      x=0;
      y+=size1;
    }
    chr++;
  }
}

//m^n
u32 OLED_Pow(uint8_t m,uint8_t n)
{
  u32 result=1;
  while(n--)
  {
    result*=m;
  }
  return result;
}

////Show 2 numbers
////x,y :Start coordinates  
////len :Number of digits
////size:Font size
void OLED_ShowNum(uint8_t x,uint8_t y,int num,uint8_t len,uint8_t size1)
{
  uint8_t t,temp;
  for(t=0;t<len;t++)
  {
    temp=(num/OLED_Pow(10,len-t-1))%10;
      if(temp==0)
      {
        OLED_ShowChar(x+(size1/2)*t,y,'0',size1);
      }
      else 
      {
        OLED_ShowChar(x+(size1/2)*t,y,temp+'0',size1);
      }
  }
}

//Show Chinese characters
//x,y:Start coordinates
//num:The serial number corresponding to the Chinese character
//Mode-picking, column-row
void OLED_ShowChinese(uint8_t x,uint8_t y,const uint8_t num,uint8_t size1)
{
  uint8_t i,m,n=0,temp,chr1;
  uint8_t x0=x,y0=y;
  uint8_t size3=size1/8;
  while(size3--)
  {
    chr1=num*size1/8+n;
    n++;
      for(i=0;i<size1;i++)
      {
        if(size1==16)
            {temp=pgm_read_byte(&Hzk1[chr1][i]);}//Call 16*16 font
        else if(size1==24)
            {temp=pgm_read_byte(&Hzk2[chr1][i]);}//Call 24*24 font
        else if(size1==32)       
            {temp=pgm_read_byte(&Hzk3[chr1][i]);}//Call 32*32 font 
        else if(size1==64)
            {temp=pgm_read_byte(&Hzk4[chr1][i]);}//Call 64*64 font  
        else return;
              
            for(m=0;m<8;m++)
              {
                if(temp&0x01)OLED_DrawPoint(x,y);
                else OLED_ClearPoint(x,y);
                temp>>=1;
                y++;
              }
              x++;
              if((x-x0)==size1)
              {x=x0;y0=y0+8;}
              y=y0;
       }
  }
}

//Configure the starting point of writing data
void OLED_WR_BP(uint8_t x,uint8_t y)
{
  OLED_WR_Byte(0xb0+y,OLED_CMD);//Set the start address of the line
  OLED_WR_Byte(((x&0xf0)>>4)|0x10,OLED_CMD);
  OLED_WR_Byte((x&0x0f),OLED_CMD);
}

//x0,y0：Start coordinates
//x1,y1：End coordinates
//BMP[]：An array of pictures to write to
void OLED_ShowPicture(uint8_t x0,uint8_t y0,uint8_t x1,uint8_t y1,const uint8_t BMP[])
{
  int j=0;
  uint8_t t;
  uint8_t x,y;
  for(y=y0;y<y1;y++)
   {
     OLED_WR_BP(x0,y);
     for(x=x0;x<x1;x++)
     {
       t=pgm_read_byte(&BMP[j++]);
       OLED_WR_Byte(t,OLED_DATA);
     }
   }
}

//Initialization of OLED
void OLED_Init(void)
{
  pinMode(scl,OUTPUT); 
  pinMode(sda,OUTPUT); 
  delay(200);
  
  OLED_WR_Byte(0xAE,OLED_CMD);//--turn off oled panel
  OLED_WR_Byte(0x00,OLED_CMD);//---set low column address
  OLED_WR_Byte(0x10,OLED_CMD);//---set high column address
  OLED_WR_Byte(0x40,OLED_CMD);//--set start line address  Set Mapping RAM Display Start Line (0x00~0x3F)
  OLED_WR_Byte(0x81,OLED_CMD);//--set contrast control register
  OLED_WR_Byte(0xCF,OLED_CMD);// Set SEG Output Current Brightness
  OLED_WR_Byte(0xA1,OLED_CMD);//--Set SEG/Column Mapping     0xa0左右反置 0xa1正常
  OLED_WR_Byte(0xC8,OLED_CMD);//Set COM/Row Scan Direction   0xc0上下反置 0xc8正常
  OLED_WR_Byte(0xA6,OLED_CMD);//--set normal display
  OLED_WR_Byte(0xA8,OLED_CMD);//--set multiplex ratio(1 to 64)
  OLED_WR_Byte(0x1f,OLED_CMD);//--1/64 duty
  OLED_WR_Byte(0xD3,OLED_CMD);//-set display offset Shift Mapping RAM Counter (0x00~0x3F)
  OLED_WR_Byte(0x00,OLED_CMD);//-not offset
  OLED_WR_Byte(0xd5,OLED_CMD);//--set display clock divide ratio/oscillator frequency
  OLED_WR_Byte(0x80,OLED_CMD);//--set divide ratio, Set Clock as 100 Frames/Sec
  OLED_WR_Byte(0xD9,OLED_CMD);//--set pre-charge period
  OLED_WR_Byte(0xF1,OLED_CMD);//Set Pre-Charge as 15 Clocks & Discharge as 1 Clock
  OLED_WR_Byte(0xDA,OLED_CMD);//--set com pins hardware configuration
  OLED_WR_Byte(0x00,OLED_CMD);
  OLED_WR_Byte(0xDB,OLED_CMD);//--set vcomh
  OLED_WR_Byte(0x40,OLED_CMD);//Set VCOM Deselect Level
  OLED_WR_Byte(0x20,OLED_CMD);//-Set Page Addressing Mode (0x00/0x01/0x02)
  OLED_WR_Byte(0x02,OLED_CMD);//
  OLED_WR_Byte(0x8D,OLED_CMD);//--set Charge Pump enable/disable
  OLED_WR_Byte(0x14,OLED_CMD);//--set(0x10) disable
  OLED_WR_Byte(0xA4,OLED_CMD);// Disable Entire Display On (0xa4/0xa5)
  OLED_WR_Byte(0xA6,OLED_CMD);// Disable Inverse Display On (0xa6/a7) 
  OLED_WR_Byte(0xAF,OLED_CMD);
  OLED_Clear();
}
